import java.util.ArrayList;
import java.util.Random;

public class SAPI {
	public static class Dungeon {
		private static ArrayList<PairFloatDungeonLoot> loot = new ArrayList<PairFloatDungeonLoot>();
		private static ArrayList<PairFloatString> mobs = new ArrayList<PairFloatString>();

		public static void addDungeonLoot(DungeonLoot dl) {
			addDungeonLoot(dl,1f);
		}

		public static void addDungeonLoot(DungeonLoot dl, float commonness) {
			loot.add(new PairFloatDungeonLoot(commonness,dl));
		}

		public static void addMob(String mob) {
			addMob(mob,1f);
		}

		public static void addMob(String mob, float commonness) {
			mobs.add(new PairFloatString(commonness,mob));
		}

		private static float lootGetMaxRandom() {
			float ret = 0f;
			for (PairFloatDungeonLoot pair : loot)
				ret += pair.f;
			return ret;
		}

		private static DungeonLoot lootGetFromRandomFloat(float random) {
			for (PairFloatDungeonLoot pair : loot) {
				if (random < pair.f) return pair.dl;
				random -= pair.f;
			}
			return null;
		}

		public static DungeonLoot lootGet(Random rnd) {
			return lootGetFromRandomFloat(rnd.nextFloat() * lootGetMaxRandom());
		}

		private static float mobGetMaxRandom() {
			float ret = 0f;
			for (PairFloatString pair : mobs)
				ret += pair.f;
			return ret;
		}

		private static String mobGetFromRandomFloat(float random) {
			for (PairFloatString pair : mobs) {
				if (random < pair.f) return pair.s;
				random -= pair.f;
			}
			return null;
		}

		public static String mobGet(Random rnd) {
			return mobGetFromRandomFloat(rnd.nextFloat() * mobGetMaxRandom());
		}
	}

	public static class Achievements {
		private static ArrayList<AchievementPage> pages = new ArrayList<AchievementPage>();
		private static ArrayList<Integer> hidden = new ArrayList<Integer>();
		private static int current = 0;
		protected static AchievementPage defaultPage = new AchievementPage();

		public static void addPage(AchievementPage page) {
			pages.add(page);
		}

		public static AchievementPage getPage(ws achievement) {
			if (achievement == null) return null;
			for (AchievementPage page : pages) if (page.list.contains(achievement.a)) return page;
			return defaultPage;
		}

		public static void hide(ws... achievements) {
			for (ws a : achievements) hidden.add(a.a);
		}

		public static void unhide(ws... achievements) {
			for (ws a : achievements) hidden.remove((Object)a.a);
		}

		public static boolean isHidden(ws achievement) {
			for (int a : hidden) if (achievement.a == a) return true;
			return false;
		}

		public static ArrayList<AchievementPage> getPages() {
			return new ArrayList<AchievementPage>(pages);
		}

		public static AchievementPage current() {
			return pages.get(current);
		}

		public static void setPage(AchievementPage page) {
			for (int i = 0; i < pages.size(); i++) if (pages.get(i).equals(page)) {
				current = i;
				return;
			}
		}
	}

	public static class LockDifficulty {
		private static ArrayList<ILockDifficulty> mods = new ArrayList<ILockDifficulty>();

		public static void add(ILockDifficulty mod) {
			mods.add(mod);
		}

		public static String listLockingMods() {
			String s = "";
			if (mods.isEmpty()) return s;
			for (ILockDifficulty mod : mods) {
				if (!mod.iLockShouldLock()) continue;
				if (!s.isEmpty()) s += ",";
				s += mod.iLockModName();
			}
			return s;
		}
	}

	public static class NBT {
		private static ArrayList<INBT> nbt = new ArrayList<INBT>();

		public static void add(INBT inbt) {
			nbt.add(inbt);
		}

		public static ArrayList<INBT> getList() {
			return new ArrayList<INBT>(nbt);
		}
	}

	public static class Reach {
		private static ArrayList<IReach> reach = new ArrayList<IReach>();

		public static void add(IReach ireach) {
			reach.add(ireach);
		}

		public static ArrayList<IReach> getList() {
			return new ArrayList<IReach>(reach);
		}

		public static float getReach(boolean entity) {
			dk hand = ModLoader.getMinecraftInstance().h.s();
			for (IReach r : reach) if (r.iReachMatches(entity,hand)) return r.iReachGet(entity,hand);
			return entity ? 3f : 4f;
		}
	}

	public static class BlockSet {
		private static ArrayList<IBlockSet> blockset = new ArrayList<IBlockSet>();

		public static void add(IBlockSet iblockset) {
			blockset.add(iblockset);
		}

		public static ArrayList<IBlockSet> getList() {
			return new ArrayList<IBlockSet>(blockset);
		}

		public static IBlockSet getIntercept(ry world, int x, int y, int z, int blockID) {
			for (IBlockSet i : blockset) if (i.iBlockMatches(world,x,y,z,blockID)) return i;
			return null;
		}
	}

	public static class Harvest {
		private static ArrayList<IHarvest> harvest = new ArrayList<IHarvest>();

		public static void add(IHarvest iharvest) {
			harvest.add(iharvest);
		}

		public static ArrayList<IHarvest> getList() {
			return new ArrayList<IHarvest>(harvest);
		}

		public static boolean intercept(ry world, vi player, int x, int y, int z, int blockID, int metadata) {
			for (IHarvest i : harvest) if (i.iHarvest(world,player,x,y,z,blockID,metadata)) return true;
			return false;
		}
	}

	private static class PairFloatDungeonLoot {
		private float f;
		private DungeonLoot dl;

		public PairFloatDungeonLoot(float f, DungeonLoot dl) {
			this.f = f;
			this.dl = dl;
		}
	}
	private static class PairFloatString {
		private float f;
		private String s;

		public PairFloatString(float f, String s) {
			this.f = f;
			this.s = s;
		}
	}
}